/**
 * @file                                                    THERMO_K_click.c
 * @brief                   THERMO K Example
 *
 * @par
 * Example for Thermo K click board.
 *
 * Test configuration :
 * @par
 * - <b>MCU           </b> :    PIC18F87K22
 * - <b>Dev. Board    </b> :    EasyPIC PRO v7
 * - <b>Ext. Modules  </b> :    LCD 2x16 & Thermo K click
 * - <b>SW            </b> :    mikroC PRO for PIC v7.0.0
 *
 ******************************************************************************/

#include "MCP9600.h"

sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;

sbit LCD_BCK at LATG3_bit;
sbit LCD_BCK_Direction at TRISG3_bit;

char tk_status;
char txt[10];
char lab1[10];
char old_txt[10];
char tmp_data[10];
char Temp_Data_Ready;

/*******************************************************************************
* Function Write_MCP9600_Data(RegAddr)
* ------------------------------------------------------------------------------
* Overview: Function writes Data to MCP9600
* Input: Register address, data
* Output: Nothing
*******************************************************************************/
char Write_MCP9600_Data(char RegAddr, char _data)
{
  tmp_data[0] = RegAddr;                                                        // Register address
  tmp_data[1] = _data;
  I2C1_Start();
  I2C1_Wr( MCP9600_I2C_ADDR );
  I2C1_Wr( tmp_data[ 0 ] );
  I2C1_Wr( tmp_data[ 1 ] );
  I2C1_Stop();
}

/*******************************************************************************
* Function Read_MCP9600_Data(RegAddr)
* ------------------------------------------------------------------------------
* Overview: Function gets Data from MCP9600
* Input: Register address
* Output: Value from desired register
*******************************************************************************/
char Read_MCP9600_Data( char RegAddr )
{
  tmp_data[0] = RegAddr;                                                        // Register address
  I2C1_Start();
  I2C1_Wr( MCP9600_I2C_ADDR );
  I2C1_Wr( tmp_data[ 0 ] );
  I2C1_Stop();
  Delay_us( 50 );
  I2C1_Start();
  I2C1_Wr( MCP9600_I2C_ADDR | 1 );
  tmp_data[ 0 ] = I2C1_Rd( 0 );
  I2C1_Stop();
  return tmp_data[0];                                                           // Return MCP9600 Data value
}

/*******************************************************************************
* Function Read_MCP9600_Status
* ------------------------------------------------------------------------------
* Overview: Function gets Data from MCP9600
* Input: Register address
* Output: Value from desired register
*******************************************************************************/
char Read_MCP9600_Status()
{
  char DataSum;
  tmp_data[0] = MCP9600_Status;                                                 // Register address
  I2C1_Start();
  I2C1_Wr( MCP9600_I2C_ADDR );
  I2C1_Wr( tmp_data[ 0 ] );
  I2C1_Stop();
  Delay_us( 50 );
  I2C1_Start();
  I2C1_Wr( MCP9600_I2C_ADDR | 1 );
  tmp_data[ 0 ] = I2C1_Rd( 0 );
  I2C1_Stop();
  DataSum = tmp_data[0];

  return DataSum;                                                               // Return MCP9600 Data value
}

/*******************************************************************************
* Function Read_Temperature()
* ------------------------------------------------------------------------------
* Overview: Function gets Thermocouple Temperature from MCP9600
* Input: Nothing
* Output: Thermocouple Temperature
*******************************************************************************/
float Read_Temperature()
{
  float Temperature;

  tmp_data[0] = MCP9600_TH;                                                     // Thermocouple Temperature register

  I2C1_Start();
  I2C1_Wr( MCP9600_I2C_ADDR );
  I2C1_Wr( tmp_data[ 0 ] );
  I2C1_Stop();
  Delay_us( 50 );
  I2C1_Start();
  I2C1_Wr( MCP9600_I2C_ADDR | 1 );
  tmp_data[ 0 ] = I2C1_Rd( 1 );
  tmp_data[ 1 ] = I2C1_Rd( 0 );
  I2C1_Stop();

  if((tmp_data[0] & 0x80) == 0x80){                                             // TA < 0C
    tmp_data[0] = tmp_data[0] & 0x7F;                                           // Clear SIGN
    Temperature = 1024 - (tmp_data[0]*16 + tmp_data[1] / 16);
  }
  else                                                                          // TA > 0C
    Temperature = (tmp_data[0] * 16 + (float)tmp_data[1] / 16);                 // Temperature = Ambient Temperature (C)

  return Temperature;                                                           // Return Thermocouple temperature data
}

/*******************************************************************************
* Function updateLabel(char *newText, int x, int y, char* lbl)
* ------------------------------------------------------------------------------
* Overview: Function writes text on display and erase old text
* Input: Nothing
* Output: Updated text
*******************************************************************************/
void updateLabel(char *newText, int x, int y, char* lbl)
{
  Lcd_Out( 2, 5, newText );
}

//******************************************************************************
// Main function
//******************************************************************************
void main()
{
  char res;
  
  ANCON0 = 0;                        											// Configure PORTB pins as digital
  ANCON1 = 0;
  ANCON2 = 0;
  LCD_BCK_Direction = 0;
  LCD_BCK = 1;

  Lcd_Init();                        											// Initialize LCD
  Lcd_Cmd(_LCD_CLEAR);               											// Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);
  Lcd_Out( 1, 2, "Thermo K click" );
  I2C1_Init( 100000 );
  Delay_ms( 200 );                                                              // Delay 200 ms

  while(1){
    tk_status = &Read_MCP9600_Status;
    Temp_Data_Ready = ( tk_status & 0x40 );

    if (Temp_Data_Ready){                                                       // Check is Temperature Data ready
      sprintf(txt, "%2.1f C", Read_Temperature());                              // Format Temperature Data and store it to txt
      res = strcmp(txt,old_txt);                                                // Compare old_txt and txt

      if(res != 0){                                                             // If old_txt and txt are not equal
        updateLabel(txt, 135, 90, lab1);                                        // Write temperature value on display
        strcpy(old_txt,txt);                                                    // Copy txt to old_txt string array
      }
    }
    Delay_ms(500);
  }
}
